@@ -1,9 +1,11 @@
 """Module that actually runs the games."""
 
 # pylint: disable=too-many-public-methods
+import json
 import os
 import shlex
 import shutil
+import signal
 import subprocess
 import time
 from gettext import gettext as _
@@ -16,13 +18,11 @@ from lutris.config import LutrisConfig
 from lutris.database import categories as categories_db
 from lutris.database import games as games_db
 from lutris.database import sql
-from lutris.discord import DiscordPresence
 from lutris.exceptions import GameConfigError, watch_lutris_errors
 from lutris.gui import dialogs
 from lutris.runner_interpreter import export_bash_script, get_launch_parameters
 from lutris.runners import InvalidRunner, import_runner, wine
-from lutris.settings import DEFAULT_DISCORD_CLIENT_ID
-from lutris.util import audio, jobs, strings, system, xdgshortcuts
+from lutris.util import audio, extract, jobs, linux, strings, system, xdgshortcuts
 from lutris.util.display import (
     DISPLAY_MANAGER, SCREEN_SAVER_INHIBITOR, disable_compositing, enable_compositing, restore_gamma
 )
@@ -30,7 +30,9 @@ from lutris.util.graphics.xephyr import get_xephyr_command
 from lutris.util.graphics.xrandr import turn_off_except
 from lutris.util.linux import LINUX_SYSTEM
 from lutris.util.log import LOG_BUFFERS, logger
+from lutris.util.process import Process
 from lutris.util.timer import Timer
+from lutris.util.yaml import write_yaml_to_file
 
 HEARTBEAT_DELAY = 2000
 
@@ -39,12 +41,15 @@ class Game(GObject.Object):
     """This class takes cares of loading the configuration for a game
        and running it.
     """
+
+    now_playing_path = os.path.join(settings.CACHE_DIR, "now-playing.txt")
+
     STATE_STOPPED = "stopped"
     STATE_LAUNCHING = "launching"
     STATE_RUNNING = "running"
 
     __gsignals__ = {
-        "game-error": (GObject.SIGNAL_RUN_FIRST, None, (str, )),
+        "game-error": (GObject.SIGNAL_RUN_FIRST, None, (object, )),
         "game-launch": (GObject.SIGNAL_RUN_FIRST, None, ()),
         "game-start": (GObject.SIGNAL_RUN_FIRST, None, ()),
         "game-started": (GObject.SIGNAL_RUN_FIRST, None, ()),
@@ -53,6 +58,8 @@ class Game(GObject.Object):
         "game-removed": (GObject.SIGNAL_RUN_FIRST, None, ()),
         "game-updated": (GObject.SIGNAL_RUN_FIRST, None, ()),
         "game-install": (GObject.SIGNAL_RUN_FIRST, None, ()),
+        "game-install-update": (GObject.SIGNAL_RUN_FIRST, None, ()),
+        "game-install-dlc": (GObject.SIGNAL_RUN_FIRST, None, ()),
         "game-installed": (GObject.SIGNAL_RUN_FIRST, None, ()),
     }
 
@@ -64,11 +71,11 @@ class Game(GObject.Object):
 
         # Load attributes from database
         game_data = games_db.get_game_by_field(game_id, "id")
+
         self.slug = game_data.get("slug") or ""
         self.runner_name = game_data.get("runner") or ""
         self.directory = game_data.get("directory") or ""
         self.name = game_data.get("name") or ""
-
         self.game_config_id = game_data.get("configpath") or ""
         self.is_installed = bool(game_data.get("installed") and self.game_config_id)
         self.is_hidden = bool(game_data.get("hidden"))
@@ -79,12 +86,14 @@ class Game(GObject.Object):
         self.has_custom_icon = bool(game_data.get("has_custom_icon"))
         self.service = game_data.get("service")
         self.appid = game_data.get("service_id")
-        self.discord_presence = DiscordPresence()
         self.playtime = game_data.get("playtime") or 0.0
 
         if self.game_config_id:
             self.load_config()
+        self.game_uuid = None
         self.game_thread = None
+        self.antimicro_thread = None
+        self.prelaunch_pids = []
         self.prelaunch_executor = None
         self.heartbeat = None
         self.killswitch = None
@@ -101,11 +110,16 @@ class Game(GObject.Object):
         return self.__str__()
 
     def __str__(self):
-        value = self.name
+        value = self.name or "Game (no name)"
         if self.runner_name:
             value += " (%s)" % self.runner_name
         return value
 
+    @property
+    def is_updatable(self):
+        """Return whether the game can be upgraded"""
+        return self.service == "gog"
+
     @property
     def is_favorite(self):
         """Return whether the game is in the user's favorites"""
@@ -129,20 +143,16 @@ class Game(GObject.Object):
         categories_db.remove_category_from_game(self.id, favorite["id"])
         self.emit("game-updated")
 
-    def hide(self):
+    def set_hidden(self, is_hidden):
         """Do not show this game in the UI"""
-        self.is_hidden = True
-        self.save()
-
-    def unhide(self):
-        """Remove the game from hidden games"""
-        self.is_hidden = False
+        self.is_hidden = is_hidden
         self.save()
+        self.emit("game-updated")
 
     @property
     def log_buffer(self):
         """Access the log buffer object, creating it if necessary"""
-        _log_buffer = LOG_BUFFERS.get(self.id)
+        _log_buffer = LOG_BUFFERS.get(str(self.id))
         if _log_buffer:
             return _log_buffer
         _log_buffer = Gtk.TextBuffer()
@@ -150,7 +160,7 @@ class Game(GObject.Object):
         if self.game_thread:
             self.game_thread.set_log_buffer(self._log_buffer)
             _log_buffer.set_text(self.game_thread.stdout)
-        LOG_BUFFERS[self.id] = _log_buffer
+        LOG_BUFFERS[str(self.id)] = _log_buffer
         return _log_buffer
 
     @property
@@ -159,30 +169,29 @@ class Game(GObject.Object):
         return strings.get_formatted_playtime(self.playtime)
 
     @staticmethod
-    def show_error_message(message):
-        """Display an error message based on the runner's output."""
-        if message["error"] == "CUSTOM":
-            message_text = message["text"].replace("&", "&amp;")
-            dialogs.ErrorDialog(message_text)
-        elif message["error"] == "RUNNER_NOT_INSTALLED":
-            dialogs.ErrorDialog(_("Error the runner is not installed"))
-        elif message["error"] == "NO_BIOS":
-            dialogs.ErrorDialog(_("A bios file is required to run this game"))
-        elif message["error"] == "FILE_NOT_FOUND":
-            filename = message["file"]
+    def get_config_error(gameplay_info):
+        """Return a GameConfigError based on the runner's output."""
+        error = gameplay_info["error"]
+        if error == "CUSTOM":
+            message_text = gameplay_info["text"].replace("&", "&amp;")
+        elif error == "RUNNER_NOT_INSTALLED":
+            message_text = _("Error the runner is not installed")
+        elif error == "NO_BIOS":
+            message_text = _("A bios file is required to run this game")
+        elif error == "FILE_NOT_FOUND":
+            filename = gameplay_info["file"]
             if filename:
                 message_text = _("The file {} could not be found").format(filename.replace("&", "&amp;"))
             else:
                 message_text = _("This game has no executable set. The install process didn't finish properly.")
-            dialogs.ErrorDialog(message_text)
-        elif message["error"] == "NOT_EXECUTABLE":
-            message_text = message["file"].replace("&", "&amp;")
-            dialogs.ErrorDialog(_("The file %s is not executable") % message_text)
-        elif message["error"] == "PATH_NOT_SET":
-            message_text = _("The path '%s' is not set. please set it in the options.") % message["path"]
-            dialogs.ErrorDialog(message_text)
+        elif error == "NOT_EXECUTABLE":
+            file = gameplay_info["file"].replace("&", "&amp;")
+            message_text = _("The file %s is not executable") % file
+        elif error == "PATH_NOT_SET":
+            message_text = _("The path '%s' is not set. please set it in the options.") % gameplay_info["path"]
         else:
-            dialogs.ErrorDialog(_("Unhandled error: %s") % message["error"])
+            message_text = _("Unhandled error: %s") % gameplay_info["error"]
+        return GameConfigError(message_text)
 
     def get_browse_dir(self):
         """Return the path to open with the Browse Files action."""
@@ -202,16 +211,6 @@ class Game(GObject.Object):
             return
         self.config = LutrisConfig(runner_slug=self.runner_name, game_config_id=self.game_config_id)
         self.runner = self._get_runner()
-        if self.discord_presence.available:
-            self.discord_presence.client_id = (
-                self.config.system_config.get("discord_client_id") or DEFAULT_DISCORD_CLIENT_ID
-            )
-            self.discord_presence.game_name = (self.config.system_config.get("discord_custom_game_name") or self.name)
-            self.discord_presence.show_runner = self.config.system_config.get("discord_show_runner", True)
-            self.discord_presence.runner_name = (
-                self.config.system_config.get("discord_custom_runner_name") or self.runner_name
-            )
-            self.discord_presence.rpc_enabled = self.config.system_config.get("discord_rpc_enabled", True)
 
     def set_desktop_compositing(self, enable):
         """Enables or disables compositing"""
@@ -224,11 +223,12 @@ class Game(GObject.Object):
                 disable_compositing()
                 self.compositor_disabled = True
 
-    def remove(self, delete_files=False):
+    def remove(self, delete_files=False, no_signal=False):
         """Uninstall a game
 
         Params:
             delete_files (bool): Delete the game files
+            no_signal (bool): Don't emit game-removed signal (if running in a thread)
         """
         sql.db_update(settings.PGA_DB, "games", {"installed": 0, "runner": ""}, {"id": self.id})
         if self.config:
@@ -238,6 +238,15 @@ class Game(GObject.Object):
             self.runner.remove_game_data(game_path=self.directory)
         self.is_installed = False
         self.runner = None
+        if no_signal:
+            return
+        self.emit("game-removed")
+
+    def delete(self):
+        """Completely remove a game from the library"""
+        if self.is_installed:
+            raise RuntimeError("Uninstall the game before deleting")
+        games_db.delete_game(self.id)
         self.emit("game-removed")
 
     def set_platform_from_runner(self):
@@ -247,8 +256,7 @@ class Game(GObject.Object):
             return
         self.platform = self.runner.get_platform()
         if not self.platform:
-            logger.warning("Can't get platform for runner %s", self.runner.human_name)
-
+            logger.warning("The %s runner didn't provide a platform for %s", self.runner.human_name, self)
 
     def save(self, save_config=False):
         """
@@ -285,6 +293,11 @@ class Game(GObject.Object):
 
     def is_launchable(self):
         """Verify that the current game can be launched."""
+        if not self.is_installed:
+            logger.error("%s (%s) not installed", self, self.id)
+            raise GameConfigError(_("Tried to launch a game that isn't installed."))
+        if not self.runner:
+            raise GameConfigError(_("Invalid game configuration: Missing runner"))
         if not self.runner.is_installed():
             installed = self.runner.install_dialog()
             if not installed:
@@ -293,13 +306,9 @@ class Game(GObject.Object):
         if self.runner.use_runtime():
             runtime_updater = runtime.RuntimeUpdater()
             if runtime_updater.is_updating():
-                logger.warning("Runtime updates: %s", runtime_updater.current_updates)
                 dialogs.ErrorDialog(_("Runtime currently updating"), _("Game might not work as expected"))
-        if ("wine" in self.runner_name and not wine.get_system_wine_version() and not LINUX_SYSTEM.is_flatpak):
-            # TODO find a reference to the root window or better yet a way not
-            # to have Gtk dependent code in this class.
-            root_window = None
-            dialogs.WineNotInstalledWarning(parent=root_window)
+        if ("wine" in self.runner_name and not wine.get_wine_version() and not LINUX_SYSTEM.is_flatpak):
+            dialogs.WineNotInstalledWarning(parent=None)
         return True
 
     def restrict_to_display(self, display):
@@ -330,43 +339,61 @@ class Game(GObject.Object):
     def start_xephyr(self, display=":2"):
         """Start a monitored Xephyr instance"""
         if not system.find_executable("Xephyr"):
-            raise GameConfigError("Unable to find Xephyr, install it or disable the Xephyr option")
+            raise GameConfigError(_("Unable to find Xephyr, install it or disable the Xephyr option"))
         xephyr_command = get_xephyr_command(display, self.runner.system_config)
         xephyr_thread = MonitoredCommand(xephyr_command)
         xephyr_thread.start()
         time.sleep(3)
         return display
 
+    def start_antimicrox(self, antimicro_config):
+        """Start Antimicrox with a given config path"""
+        antimicro_path = system.find_executable("antimicrox")
+        if not antimicro_path:
+            logger.warning("Antimicrox is not installed.")
+            return
+        logger.info("Starting Antic")
+        antimicro_command = [antimicro_path, "--hidden", "--tray", "--profile", antimicro_config]
+        self.antimicro_thread = MonitoredCommand(antimicro_command)
+        self.antimicro_thread.start()
+
     @staticmethod
     def set_keyboard_layout(layout):
         setxkbmap_command = ["setxkbmap", "-model", "pc101", layout, "-print"]
         xkbcomp_command = ["xkbcomp", "-", os.environ.get("DISPLAY", ":0")]
-        xkbcomp = subprocess.Popen(xkbcomp_command, stdin=subprocess.PIPE)
-        subprocess.Popen(setxkbmap_command, env=os.environ, stdout=xkbcomp.stdin).communicate()
-        xkbcomp.communicate()
+        with subprocess.Popen(xkbcomp_command, stdin=subprocess.PIPE) as xkbcomp:
+            with subprocess.Popen(setxkbmap_command, env=os.environ, stdout=xkbcomp.stdin) as setxkbmap:
+                setxkbmap.communicate()
+                xkbcomp.communicate()
 
-    def start_prelaunch_command(self):
+    def start_prelaunch_command(self, wait_for_completion=False):
         """Start the prelaunch command specified in the system options"""
         prelaunch_command = self.runner.system_config.get("prelaunch_command")
         command_array = shlex.split(prelaunch_command)
         if not system.path_exists(command_array[0]):
             logger.warning("Command %s not found", command_array[0])
             return
-        self.prelaunch_executor = MonitoredCommand(
-            command_array,
-            include_processes=[os.path.basename(command_array[0])],
-            env=self.game_runtime_config["env"],
-            cwd=self.directory,
-        )
-        self.prelaunch_executor.start()
-        logger.info("Running %s in the background", prelaunch_command)
+        env = self.game_runtime_config["env"]
+        if wait_for_completion:
+            logger.info("Prelauch command: %s, waiting for completion", prelaunch_command)
+            # Monitor the prelaunch command and wait until it has finished
+            system.execute(command_array, env=env, cwd=self.directory)
+        else:
+            logger.info("Prelaunch command %s launched in the background", prelaunch_command)
+            self.prelaunch_executor = MonitoredCommand(
+                command_array,
+                include_processes=[os.path.basename(command_array[0])],
+                env=env,
+                cwd=self.directory,
+            )
+            self.prelaunch_executor.start()
 
     def get_terminal(self):
         """Return the terminal used to run the game into or None if the game is not run from a terminal.
         Remember that only games using text mode should use the terminal.
         """
         if self.runner.system_config.get("terminal"):
-            terminal = self.runner.system_config.get("terminal_app", system.get_default_terminal())
+            terminal = self.runner.system_config.get("terminal_app", linux.get_default_terminal())
             if terminal and not system.find_executable(terminal):
                 raise GameConfigError(_("The selected terminal application could not be launched:\n%s") % terminal)
             return terminal
@@ -384,12 +411,38 @@ class Game(GObject.Object):
         """Return the information provided by a runner's play method.
         Checks for possible errors.
         """
-        gameplay_info = self.runner.play()
-        if "error" in gameplay_info:
-            self.show_error_message(gameplay_info)
+        if not self.runner:
+            logger.warning("Trying to launch %s without a runner", self)
             self.state = self.STATE_STOPPED
             self.emit("game-stop")
-            return
+            return {}
+        gameplay_info = self.runner.play()
+        if "error" in gameplay_info:
+            self.state = self.STATE_STOPPED
+            self.emit("game-stop")
+            raise self.get_config_error(gameplay_info)
+
+        if self.config.game_level.get("game", {}).get("launch_configs"):
+            configs = self.config.game_level["game"]["launch_configs"]
+            dlg = dialogs.LaunchConfigSelectDialog(self, configs)
+            if not dlg.confirmed:
+                self.state = self.STATE_STOPPED
+                self.emit("game-stop")
+                return {}
+
+            if dlg.config_index:
+                config = configs[dlg.config_index - 1]
+                if "command" not in gameplay_info:
+                    logger.debug("No command in %s", gameplay_info)
+                    logger.debug(config)
+                    self.state = self.STATE_STOPPED
+                    self.emit("game-stop")
+                    return {}
+
+                gameplay_info["command"] = [gameplay_info["command"][0], config["exe"]]
+                if config.get("args"):
+                    gameplay_info["command"] += strings.split_arguments(config["args"])
+
         return gameplay_info
 
     @watch_lutris_errors
@@ -420,17 +473,14 @@ class Game(GObject.Object):
         }
 
         # Audio control
-
         if self.runner.system_config.get("reset_pulse"):
             audio.reset_pulse()
 
         # Input control
-
         if self.runner.system_config.get("use_us_layout"):
             self.set_keyboard_layout("us")
 
         # Display control
-
         self.original_outputs = DISPLAY_MANAGER.get_config()
 
         if self.runner.system_config.get("disable_compositor"):
@@ -452,36 +502,33 @@ class Game(GObject.Object):
         if xephyr != "off":
             env["DISPLAY"] = self.start_xephyr()
 
-        # Execution control
+        antimicro_config = self.runner.system_config.get("antimicro_config")
+        if system.path_exists(antimicro_config):
+            self.start_antimicrox(antimicro_config)
 
+        # Execution control
         self.killswitch = self.get_killswitch()
 
         if self.runner.system_config.get("prelaunch_command"):
-            self.start_prelaunch_command()
+            self.start_prelaunch_command(self.runner.system_config.get("prelaunch_wait"))
 
-        if self.runner.system_config.get("prelaunch_wait"):
-            # Monitor the prelaunch command and wait until it has finished
-            self.heartbeat = GLib.timeout_add(HEARTBEAT_DELAY, self.prelaunch_beat)
-        else:
-            self.start_game()
+        self.start_game()
 
+    @watch_lutris_errors
     def launch(self):
         """Request launching a game. The game may not be installed yet."""
-        if not self.is_installed:
-            self.emit("game-install")
-            return
-        self.load_config()  # Reload the config before launching it.
-
-        if self.id in LOG_BUFFERS:  # Reset game logs on each launch
-            LOG_BUFFERS.pop(self.id)
-
-        if not self.runner:
-            dialogs.ErrorDialog(_("Invalid game configuration: Missing runner"))
-            return
         if not self.is_launchable():
             logger.error("Game is not launchable")
             return
+
+        self.load_config()  # Reload the config before launching it.
+
+        if str(self.id) in LOG_BUFFERS:  # Reset game logs on each launch
+            log_buffer = LOG_BUFFERS[str(self.id)]
+            log_buffer.delete(log_buffer.get_start_iter(), log_buffer.get_end_iter())
+
         self.state = self.STATE_LAUNCHING
+        self.prelaunch_pids = system.get_running_pid_list()
         self.emit("game-start")
         jobs.AsyncCall(self.runner.prelaunch, self.configure_game)
 
@@ -499,16 +546,94 @@ class Game(GObject.Object):
         )
         if hasattr(self.runner, "stop"):
             self.game_thread.stop_func = self.runner.stop
+        self.game_uuid = self.game_thread.env["LUTRIS_GAME_UUID"]
         self.game_thread.start()
         self.timer.start()
         self.state = self.STATE_RUNNING
         self.emit("game-started")
         self.heartbeat = GLib.timeout_add(HEARTBEAT_DELAY, self.beat)
+        with open(self.now_playing_path, "w", encoding="utf-8") as np_file:
+            np_file.write(self.name)
+
+    def force_stop(self):
+        # If force_stop_game fails, wait a few seconds and try SIGKILL on any survivors
+        self.runner.force_stop_game(self)
+        if self.get_stop_pids():
+            self.force_kill_delayed()
+        else:
+            self.stop_game()
+
+    def force_kill_delayed(self, death_watch_seconds=5, death_watch_interval_seconds=.5):
+        """Forces termination of a running game, but only after a set time has elapsed;
+        Invokes stop_game() when the game is dead."""
+
+        def death_watch():
+            """Wait for the processes to die; returns True if do they all did."""
+            for _n in range(int(death_watch_seconds / death_watch_interval_seconds)):
+                time.sleep(death_watch_interval_seconds)
+                if not self.get_stop_pids():
+                    return True
+            return False
+
+        def death_watch_cb(all_died, error):
+            """Called after the death watch to more firmly kill any survivors."""
+            if error:
+                dialogs.ErrorDialog(str(error))
+            elif not all_died:
+                self.kill_processes(signal.SIGKILL)
+            # If we still can't kill everything, we'll still say we stopped it.
+            self.stop_game()
+
+        jobs.AsyncCall(death_watch, death_watch_cb)
+
+    def kill_processes(self, sig):
+        """Sends a signal to a process list, logging errors."""
+        pids = self.get_stop_pids()
+
+        for pid in pids:
+            try:
+                os.kill(int(pid), sig)
+            except ProcessLookupError as ex:
+                logger.debug("Failed to kill game process: %s", ex)
+
+    def get_stop_pids(self):
+        """Finds the PIDs of processes that need killin'!"""
+        pids = self.get_game_pids()
+        if self.game_thread and self.game_thread.game_process:
+            pids.add(self.game_thread.game_process.pid)
+        return pids
+
+    def get_game_pids(self):
+        """Return a list of processes belonging to the Lutris game"""
+        new_pids = self.get_new_pids()
+        game_pids = []
+        game_folder = self.runner.game_path or ""
+        for pid in new_pids:
+            cmdline = Process(pid).cmdline or ""
+            # pressure-vessel: This could potentially pick up PIDs not started by lutris?
+            if game_folder in cmdline or "pressure-vessel" in cmdline:
+                game_pids.append(pid)
+        return set(game_pids + [
+            pid for pid in new_pids
+            if Process(pid).environ.get("LUTRIS_GAME_UUID") == self.game_uuid
+        ])
+
+    def get_new_pids(self):
+        """Return list of PIDs started since the game was launched"""
+        return set(system.get_running_pid_list()) - set(self.prelaunch_pids)
 
     def stop_game(self):
         """Cleanup after a game as stopped"""
+        duration = self.timer.duration
+        logger.debug("%s has run for %s seconds", self, duration)
+        if duration < 5:
+            logger.warning("The game has run for a very short time, did it crash?")
+            # Inspect why it could have crashed
+
         self.state = self.STATE_STOPPED
         self.emit("game-stop")
+        if os.path.exists(self.now_playing_path):
+            os.unlink(self.now_playing_path)
         if not self.timer.finished:
             self.timer.end()
             self.playtime += self.timer.duration / 3600
@@ -530,14 +655,15 @@ class Game(GObject.Object):
         # The killswitch file should be set to a device (ie. /dev/input/js0)
         # When that device is unplugged, the game is forced to quit.
         killswitch_engage = self.killswitch and not system.path_exists(self.killswitch)
-        if not self.game_thread.is_running or killswitch_engage:
+        if killswitch_engage:
+            logger.warning("File descriptor no longer present, force quit the game")
+            self.force_stop()
+            return False
+        game_pids = self.get_game_pids()
+        if not self.game_thread.is_running and not game_pids:
             logger.debug("Game thread stopped")
             self.on_game_quit()
             return False
-
-        if self.discord_presence.available:
-            self.discord_presence.update_discord_rich_presence()
-
         return True
 
     def stop(self):
@@ -578,9 +704,6 @@ class Game(GObject.Object):
                 )
                 postexit_thread.start()
 
-        if self.discord_presence.available:
-            self.discord_presence.clear_discord_rich_presence()
-
         quit_time = time.strftime("%a, %d %b %Y %H:%M:%S", time.localtime())
         logger.debug("%s stopped at %s", self.name, quit_time)
         self.lastplayed = int(time.time())
@@ -588,6 +711,9 @@ class Game(GObject.Object):
 
         os.chdir(os.path.expanduser("~"))
 
+        if self.antimicro_thread:
+            self.antimicro_thread.stop()
+
         if self.resolution_changed or self.runner.system_config.get("reset_desktop"):
             DISPLAY_MANAGER.set_resolution(self.original_outputs)
 
@@ -599,7 +725,8 @@ class Game(GObject.Object):
             self.screen_saver_inhibitor_cookie = None
 
         if self.runner.system_config.get("use_us_layout"):
-            subprocess.Popen(["setxkbmap"], env=os.environ).communicate()
+            with subprocess.Popen(["setxkbmap"], env=os.environ) as setxkbmap:
+                setxkbmap.communicate()
 
         if self.runner.system_config.get("restore_gamma"):
             restore_gamma()
@@ -625,6 +752,7 @@ class Game(GObject.Object):
         """Output the launch argument in a bash script"""
         gameplay_info = self.get_gameplay_info()
         if not gameplay_info:
+            logger.error("Unable to retrieve game information for %s. Can't write a script", self)
             return
         export_bash_script(self.runner, gameplay_info, script_path)
 
@@ -643,13 +771,13 @@ class Game(GObject.Object):
             logger.info("Previous location wasn't set. Cannot continue moving")
             return target_directory
 
-        with open(self.config.game_config_path) as config_file:
+        with open(self.config.game_config_path, encoding='utf-8') as config_file:
             for line in config_file.readlines():
                 if target_directory in line:
                     new_config += line
                 else:
                     new_config += line.replace(old_location, target_directory)
-        with open(self.config.game_config_path, "w") as config_file:
+        with open(self.config.game_config_path, "w", encoding='utf-8') as config_file:
             config_file.write(new_config)
 
         if not system.path_exists(old_location):
@@ -666,3 +794,72 @@ class Game(GObject.Object):
                 old_location, new_location, ex
             )
         return target_directory
+
+
+def export_game(slug, dest_dir):
+    """Export a full game folder along with some lutris metadata"""
+
+    # List of runner where we know for sure that 1 folder = 1 game.
+    # For runners that handle ROMs, we have to handle this more finely.
+    # There is likely more than one game in a ROM folder but a ROM
+    # might have several files (like a bin/cue, or a multi-disk game)
+    exportable_runners = [
+        "linux",
+        "wine",
+        "dosbox",
+        "scummvm",
+    ]
+
+    db_game = games_db.get_game_by_field(slug, "slug")
+    if db_game["runner"] not in exportable_runners:
+        raise RuntimeError("Game %s can't be exported." % db_game["name"])
+    if not db_game["directory"]:
+        raise RuntimeError("No game directory set. Could we guess it?")
+
+    game = Game(db_game["id"])
+    db_game["config"] = game.config.game_level
+    game_path = db_game["directory"]
+    config_path = os.path.join(db_game["directory"], "%s.lutris" % slug)
+    with open(config_path, "w", encoding="utf-8") as config_file:
+        json.dump(db_game, config_file, indent=2)
+    archive_path = os.path.join(dest_dir, "%s.7z" % slug)
+    _7zip_path = os.path.join(settings.RUNTIME_DIR, "p7zip/7z")
+    command = [_7zip_path, "a", archive_path, game_path]
+    return_code = subprocess.call(command)
+    if return_code != 0:
+        print("Creating of archive in %s failed with return code %s" % (archive_path, return_code))
+
+
+def import_game(file_path, dest_dir):
+    """Import a game in Lutris"""
+    if not os.path.exists(file_path):
+        raise RuntimeError("No file %s" % file_path)
+    if not os.path.isdir(dest_dir):
+        os.makedirs(dest_dir)
+    original_file_list = set(os.listdir(dest_dir))
+    extract.extract_7zip(file_path, dest_dir)
+    new_file_list = set(os.listdir(dest_dir))
+    new_dir = list(new_file_list - original_file_list)[0]
+    game_dir = os.path.join(dest_dir, new_dir)
+    game_config = [f for f in os.listdir(game_dir) if f.endswith(".lutris")][0]
+    with open(os.path.join(game_dir, game_config), encoding="utf-8") as config_file:
+        lutris_config = json.load(config_file)
+    # old_dir = lutris_config["directory"]
+    config_filename = os.path.join(settings.CONFIG_DIR, "games/%s.yml" % lutris_config["configpath"])
+    write_yaml_to_file(lutris_config["config"], config_filename)
+    game_id = games_db.add_or_update(
+        name=lutris_config["name"],
+        runner=lutris_config["runner"],
+        slug=lutris_config["slug"],
+        platform=lutris_config["platform"],
+        directory=game_dir,
+        installed=lutris_config["installed"],
+        year=lutris_config["year"],
+        lastplayed=lutris_config["lastplayed"],
+        configpath=lutris_config["configpath"],
+        playtime=lutris_config["playtime"],
+        hidden=lutris_config["hidden"],
+        service=lutris_config["service"],
+        service_id=lutris_config["service_id"],
+    )
+    print("Added game with ID %s" % game_id)
