@@ -1,14 +1,11 @@
 """Various utilities using the GObject framework"""
-# Standard Library
 import array
 import os
 
-# Third Party Libraries
 from gi.repository import Gdk, GdkPixbuf, Gio, GLib, Gtk
 
-# Lutris Modules
 from lutris import settings
-from lutris.util import datapath, resources, system
+from lutris.util import datapath, system
 from lutris.util.log import logger
 
 try:
@@ -16,17 +13,8 @@ try:
 except ImportError:
     Image = None
 
-BANNER_SIZE = (184, 69)
-BANNER_SMALL_SIZE = (120, 45)
 ICON_SIZE = (32, 32)
-ICON_SMALL_SIZE = (20, 20)
-
-IMAGE_SIZES = {
-    "icon_small": ICON_SMALL_SIZE,
-    "icon": ICON_SIZE,
-    "banner_small": BANNER_SMALL_SIZE,
-    "banner": BANNER_SIZE,
-}
+BANNER_SIZE = (184, 69)
 
 
 def get_main_window(widget):
@@ -51,23 +39,51 @@ def open_uri(uri):
         system.execute(["xdg-open", uri])
 
 
-def get_pixbuf(image, size, fallback=None):
+def get_pixbuf(image, size, fallback=None, is_installed=True):
     """Return a pixbuf from file `image` at `size` or fallback to `fallback`"""
-    width, heigth = size
-    if system.path_exists(image):
+    width, height = size
+    pixbuf = None
+    if system.path_exists(image, exclude_empty=True):
         try:
-            return GdkPixbuf.Pixbuf.new_from_file_at_size(image, width, heigth)
+            pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(image, width, height)
+            pixbuf = pixbuf.scale_simple(width, height, GdkPixbuf.InterpType.NEAREST)
         except GLib.GError:
             logger.error("Unable to load icon from image %s", image)
-    if system.path_exists(fallback):
-        return GdkPixbuf.Pixbuf.new_from_file_at_size(fallback, width, heigth)
-    if image and not image.startswith("/"):
-        return get_stock_icon(image, width)
-    return None
+    else:
+        if not fallback:
+            fallback = get_default_icon(size)
+        if system.path_exists(fallback):
+            pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(fallback, width, height)
+    if is_installed and pixbuf:
+        pixbuf = pixbuf.scale_simple(width, height, GdkPixbuf.InterpType.NEAREST)
+        return pixbuf
+    overlay = os.path.join(datapath.get(), "media/unavailable.png")
+    transparent_pixbuf = get_overlay(overlay, size).copy()
+    if pixbuf:
+        pixbuf.composite(
+            transparent_pixbuf,
+            0,
+            0,
+            size[0],
+            size[1],
+            0,
+            0,
+            1,
+            1,
+            GdkPixbuf.InterpType.NEAREST,
+            100,
+        )
+    return transparent_pixbuf
+
+
+def has_stock_icon(name):
+    """This tests if a GTK stock icon is known; if not we can try a fallback."""
+    theme = Gtk.IconTheme.get_default()
+    return theme.has_icon(name)
 
 
 def get_stock_icon(name, size):
-    """Return a picxbuf from a stock icon name"""
+    """Return a pixbuf from a stock icon name"""
     theme = Gtk.IconTheme.get_default()
     try:
         return theme.load_icon(name, size, Gtk.IconLookupFlags.GENERIC_FALLBACK)
@@ -86,9 +102,8 @@ def get_icon(icon_name, icon_format="image", size=None, icon_type="runner"):
     icon_type -- Retrieve either a 'runner' or 'platform' icon (default 'runner')
     """
     filename = icon_name.lower().replace(" ", "") + ".png"
-    icon_path = os.path.join(datapath.get(), "media/" + icon_type + "_icons", filename)
+    icon_path = os.path.join(settings.RUNTIME_DIR, "icons/hicolor/64x64/apps", filename)
     if not os.path.exists(icon_path):
-        logger.error("Unable to find icon '%s'", icon_path)
         return None
     if icon_format == "image":
         icon = Gtk.Image()
@@ -109,43 +124,14 @@ def get_overlay(overlay_path, size):
     return transparent_pixbuf
 
 
-def get_pixbuf_for_game(game_slug, icon_type, is_installed=True):
-    if icon_type.startswith("banner"):
-        default_icon_path = os.path.join(datapath.get(), "media/default_banner.png")
-        icon_path = resources.get_banner_path(game_slug)
-    elif icon_type.startswith("icon"):
-        default_icon_path = os.path.join(datapath.get(), "media/default_icon.png")
-        icon_path = resources.get_icon_path(game_slug)
-    else:
-        logger.error("Invalid icon type '%s'", icon_type)
-        return None
-
-    size = IMAGE_SIZES[icon_type]
-
-    pixbuf = get_pixbuf(icon_path, size, fallback=default_icon_path)
-    if not is_installed:
-        unavailable_game_overlay = os.path.join(datapath.get(), "media/unavailable.png")
-        transparent_pixbuf = get_overlay(unavailable_game_overlay, size).copy()
-        pixbuf.composite(
-            transparent_pixbuf,
-            0,
-            0,
-            size[0],
-            size[1],
-            0,
-            0,
-            1,
-            1,
-            GdkPixbuf.InterpType.NEAREST,
-            100,
-        )
-        return transparent_pixbuf
-    return pixbuf
+def get_default_icon(size):
+    if size[0] == size[1]:
+        return os.path.join(datapath.get(), "media/default_icon.png")
+    return os.path.join(datapath.get(), "media/default_banner.png")
 
 
 def convert_to_background(background_path, target_size=(320, 1080)):
     """Converts a image to a pane background"""
-
     coverart = Image.open(background_path)
     coverart = coverart.convert("RGBA")
 
@@ -175,6 +161,43 @@ def convert_to_background(background_path, target_size=(320, 1080)):
     return background
 
 
+def thumbnail_image(base_image, target_size):
+    base_width, base_height = base_image.size
+    base_ratio = base_width / base_height
+    target_width, target_height = target_size
+    target_ratio = target_width / target_height
+
+    # Resize and crop coverart
+    if base_ratio >= target_ratio:
+        width = int(base_width * (target_height / base_height))
+        height = target_height
+    else:
+        width = target_width
+        height = int(base_height * (target_width / base_width))
+    x_offset = int((width - target_width) / 2)
+    y_offset = int((height - target_height) / 2)
+    base_image = base_image.resize((width, height), resample=Image.BICUBIC)
+    base_image = base_image.crop((x_offset, y_offset, width - x_offset, height - y_offset))
+    return base_image
+
+
+def paste_overlay(base_image, overlay_image, position=0.7):
+    base_width, base_height = base_image.size
+    overlay_width, overlay_height = overlay_image.size
+    offset_x = int((base_width - overlay_width) / 2)
+    offset_y = int((base_height - overlay_height) / 2)
+    base_image.paste(
+        overlay_image, (
+            offset_x,
+            offset_y,
+            overlay_width + offset_x,
+            overlay_height + offset_y
+        ),
+        mask=overlay_image
+    )
+    return base_image
+
+
 def image2pixbuf(image):
     """Converts a PIL Image to a GDK Pixbuf"""
     image_array = array.array('B', image.tobytes())
@@ -182,30 +205,6 @@ def image2pixbuf(image):
     return GdkPixbuf.Pixbuf.new_from_data(image_array, GdkPixbuf.Colorspace.RGB, True, 8, width, height, width * 4)
 
 
-def get_pixbuf_for_panel(game_slug):
-    """Return the pixbuf for the game panel background"""
-    if Image is None:
-        # PIL is not available
-        return
-    source_path = os.path.join(settings.COVERART_PATH, "%s.jpg" % game_slug)
-    if not os.path.exists(source_path):
-        source_path = os.path.join(datapath.get(), "media/generic-panel-bg.png")
-    dest_path = os.path.join(settings.CACHE_DIR, "panel_bg.png")
-    background = convert_to_background(source_path)
-    background.save(dest_path)
-    return dest_path
-
-
-def get_builder_from_file(glade_file):
-    ui_filename = os.path.join(datapath.get(), "ui", glade_file)
-    if not os.path.exists(ui_filename):
-        raise ValueError("ui file does not exists: %s" % ui_filename)
-
-    builder = Gtk.Builder()
-    builder.add_from_file(ui_filename)
-    return builder
-
-
 def get_link_button(text):
     """Return a transparent text button for the side panels"""
     button = Gtk.Button(text, visible=True)
@@ -214,3 +213,11 @@ def get_link_button(text):
     button.get_style_context().add_class("panel-button")
     button.set_size_request(-1, 24)
     return button
+
+
+def load_icon_theme():
+    """Add the lutris icon folder to the default theme"""
+    icon_theme = Gtk.IconTheme.get_default()
+    local_theme_path = os.path.join(settings.RUNTIME_DIR, "icons")
+    if local_theme_path not in icon_theme.get_search_path():
+        icon_theme.prepend_search_path(local_theme_path)
