@@ -1,20 +1,18 @@
 """Lutris installer class"""
 import json
 import os
+from gettext import gettext as _
 
-import yaml
-
-from lutris import settings
-from lutris.config import LutrisConfig, make_game_config_id
+from lutris.config import LutrisConfig, write_game_config
 from lutris.database.games import add_or_update, get_game_by_field
-from lutris.game import Game
 from lutris.installer import AUTO_ELF_EXE, AUTO_WIN32_EXE
 from lutris.installer.errors import ScriptingError
 from lutris.installer.installer_file import InstallerFile
 from lutris.installer.legacy import get_game_launcher
 from lutris.runners import import_runner
-from lutris.services import get_services
+from lutris.services import SERVICES
 from lutris.util.game_finder import find_linux_game_executable, find_windows_game_executable
+from lutris.util.gog import convert_gog_config_to_lutris, get_gog_config_from_path, get_gog_game_path
 from lutris.util.log import logger
 
 
@@ -24,15 +22,17 @@ class LutrisInstaller:  # pylint: disable=too-many-instance-attributes
     def __init__(self, installer, interpreter, service, appid):
         self.interpreter = interpreter
         self.installer = installer
+        self.is_update = False
         self.version = installer["version"]
         self.slug = installer["slug"]
         self.year = installer.get("year")
         self.runner = installer["runner"]
         self.script = installer.get("script")
-        self.game_name = self.script.get("custom-name") or installer["name"]
+        self.game_name = installer["name"]
         self.game_slug = installer["game_slug"]
         self.service = self.get_service(initial=service)
         self.service_appid = self.get_appid(installer, initial=appid)
+        self.variables = installer.get("variables", {})
         self.files = [
             InstallerFile(self.game_slug, file_id, file_meta)
             for file_desc in self.script.get("files", [])
@@ -41,19 +41,22 @@ class LutrisInstaller:  # pylint: disable=too-many-instance-attributes
         self.requires = self.script.get("requires")
         self.extends = self.script.get("extends")
         self.game_id = self.get_game_id()
+        self.is_gog = False
 
     def get_service(self, initial=None):
         if initial:
             return initial
-        if "steam" in self.runner:
-            return get_services()["steam"]()
+        if "steam" in self.runner and "steam" in SERVICES:
+            return SERVICES["steam"]()
         version = self.version.lower()
-        if "humble" in version:
-            return get_services()["humblebundle"]()
-        if "gog" in version:
-            return get_services()["gog"]()
+        if "humble" in version and "humblebundle" in SERVICES:
+            return SERVICES["humblebundle"]()
+        if "gog" in version and "gog" in SERVICES:
+            return SERVICES["gog"]()
 
     def get_appid(self, installer, initial=None):
+        if installer.get("is_dlc"):
+            return installer.get("dlcid")
         if initial:
             return initial
         if not self.service:
@@ -76,16 +79,16 @@ class LutrisInstaller:  # pylint: disable=too-many-instance-attributes
         # If the game is in the library and uninstalled, the first installation
         # updates it
         existing_game = get_game_by_field(self.game_slug, "slug")
-        if existing_game and not existing_game["installed"]:
+        if existing_game and (self.extends or not existing_game["installed"]):
             return existing_game["id"]
 
     @property
     def creates_game_folder(self):
         """Determines if an install script should create a game folder for the game"""
-        if self.requires:
+        if self.requires or self.extends:
             # Game is an extension of an existing game, folder exists
             return False
-        if self.runner in ("steam", "winesteam"):
+        if self.runner == "steam":
             # Steam games installs in their steamapps directory
             return False
         if (
@@ -118,7 +121,7 @@ class LutrisInstaller:  # pylint: disable=too-many-instance-attributes
                 errors.append("Missing libretro core in game section")
 
         # Check that Steam games have an AppID
-        if self.runner in ("steam", "winesteam"):
+        if self.runner == "steam":
             if not self.script.get("game", {}).get("appid"):
                 errors.append("Missing appid for Steam game")
 
@@ -134,45 +137,61 @@ class LutrisInstaller:  # pylint: disable=too-many-instance-attributes
                 self.files.pop(index)
                 return file.id
 
-    def prepare_game_files(self):
+    def prepare_game_files(self, patch_version=None):
         """Gathers necessary files before iterating through them."""
         if not self.files:
+            logger.info("No files to prepare")
             return
-        if self.service:
-            if self.service.online and not self.service.is_connected():
-                logger.info("Not authenticated to %s", self.service.id)
-                return
-            installer_file_id = self.pop_user_provided_file()
-            if not installer_file_id:
-                logger.warning("Could not find a file for this service")
-                return
-            logger.info("Should install %s", self.interpreter.extras)
-            if self.service.has_extras:
-                self.service.selected_extras = self.interpreter.extras
+        if not self.service:
+            logger.debug("No service to retrieve files from")
+            return
+        if self.service.online and not self.service.is_connected():
+            logger.info("Not authenticated to %s", self.service.id)
+            return
+        installer_file_id = self.pop_user_provided_file()
+        if not installer_file_id:
+            logger.warning("Could not find a file for this service")
+            return
+        logger.info("Getting files for %s", installer_file_id)
+        if self.service.has_extras:
+            logger.info("Adding selected extras to downloads")
+            self.service.selected_extras = self.interpreter.extras
+        if patch_version:
+            # If a patch version is given download the patch files instead of the installer
+            installer_files = self.service.get_patch_files(self, installer_file_id)
+        else:
             installer_files = self.service.get_installer_files(self, installer_file_id)
-            for installer_file in installer_files:
-                self.files.append(installer_file)
-            if not installer_files:
-                # Failed to get the service game, put back a user provided file
-                self.files.insert(0, "N/A: Provider installer file")
+        for installer_file in installer_files:
+            self.files.append(installer_file)
+        if not installer_files:
+            # Failed to get the service game, put back a user provided file
+            logger.debug("Unable to get files from service. Setting %s to manual.", installer_file_id)
+            self.files.insert(0, InstallerFile(self.game_slug, installer_file_id, {
+                "url": "N/A: Provider installer file",
+                "filename": ""
+            }))
 
     def _substitute_config(self, script_config):
         """Substitute values such as $GAMEDIR in a config dict."""
         config = {}
         for key in script_config:
             if not isinstance(key, str):
-                raise ScriptingError("Game config key must be a string", key)
+                raise ScriptingError(_("Game config key must be a string"), key)
             value = script_config[key]
             if str(value).lower() == 'true':
                 value = True
             if str(value).lower() == 'false':
                 value = False
-            if isinstance(value, list):
+            if key == "launch_configs":
+                # launch configuration don't need substitutions at least for now.
+                config[key] = value
+            elif isinstance(value, list):
                 config[key] = [self.interpreter._substitute(i) for i in value]
             elif isinstance(value, dict):
                 config[key] = {k: self.interpreter._substitute(v) for (k, v) in value.items()}
             elif isinstance(value, bool):
                 config[key] = value
+
             else:
                 config[key] = self.interpreter._substitute(value)
         return config
@@ -182,6 +201,10 @@ class LutrisInstaller:  # pylint: disable=too-many-instance-attributes
         if self.requires:
             # Load the base game config
             required_game = get_game_by_field(self.requires, field="installer_slug")
+            if not required_game:
+                required_game = get_game_by_field(self.requires, field="slug")
+            if not required_game:
+                raise ValueError("No game matched '%s' on installer_slug or slug" % self.requires)
             base_config = LutrisConfig(
                 runner_slug=self.runner, game_config_id=required_game["configpath"]
             )
@@ -201,27 +224,16 @@ class LutrisInstaller:  # pylint: disable=too-many-instance-attributes
         if "game" in self.script:
             try:
                 config["game"].update(self.script["game"])
-            except ValueError:
-                raise ScriptingError("Invalid 'game' section", self.script["game"])
+            except ValueError as err:
+                raise ScriptingError(_("Invalid 'game' section"), self.script["game"]) from err
             config["game"] = self._substitute_config(config["game"])
             if AUTO_ELF_EXE in config["game"].get("exe", ""):
                 config["game"]["exe"] = find_linux_game_executable(self.interpreter.target_path,
                                                                    make_executable=True)
             elif AUTO_WIN32_EXE in config["game"].get("exe", ""):
                 config["game"]["exe"] = find_windows_game_executable(self.interpreter.target_path)
-
         return config
 
-    def write_game_config(self):
-        configpath = make_game_config_id(self.slug)
-        config_filename = os.path.join(settings.CONFIG_DIR, "games/%s.yml" % configpath)
-        config = self.get_game_config()
-        yaml_config = yaml.safe_dump(config, default_flow_style=False)
-        with open(config_filename, "w") as config_file:
-            logger.debug("Writing game config to %s", config_filename)
-            config_file.write(yaml_config)
-        return configpath
-
     def save(self):
         """Write the game configuration in the DB and config file"""
         if self.extends:
@@ -229,8 +241,16 @@ class LutrisInstaller:  # pylint: disable=too-many-instance-attributes
                 "This is an extension to %s, not creating a new game entry",
                 self.extends,
             )
-            return
-        configpath = self.write_game_config()
+            return self.game_id
+
+        if self.is_gog:
+            gog_config = get_gog_config_from_path(self.interpreter.target_path)
+            if gog_config:
+                gog_game_path = get_gog_game_path(self.interpreter.target_path)
+                lutris_config = convert_gog_config_to_lutris(gog_config, gog_game_path)
+                self.script["game"].update(lutris_config)
+
+        configpath = write_game_config(self.slug, self.get_game_config())
         runner_inst = import_runner(self.runner)()
         if self.service:
             service_id = self.service.id
@@ -243,6 +263,7 @@ class LutrisInstaller:  # pylint: disable=too-many-instance-attributes
             platform=runner_inst.get_platform(),
             directory=self.interpreter.target_path,
             installed=1,
+            hidden=0,
             installer_slug=self.slug,
             parent_slug=self.requires,
             year=self.year,
@@ -251,9 +272,7 @@ class LutrisInstaller:  # pylint: disable=too-many-instance-attributes
             service_id=self.service_appid,
             id=self.game_id,
         )
-        # This is a bit redundant but used to trigger the game-updated signal
-        game = Game(self.game_id)
-        game.save()
+        return self.game_id
 
     def get_game_launcher_config(self, game_files):
         """Game options such as exe or main_file can be added at the root of the
