@@ -36,12 +36,9 @@ class atari800(Runner):
     }
     game_options = [
         {
-            "option":
-            "main_file",
-            "type":
-            "file",
-            "label":
-            _("ROM file"),
+            "option": "main_file",
+            "type": "file",
+            "label": _("ROM file"),
             "help": _(
                 "The game data, commonly called a ROM image. \n"
                 "Supported formats: ATR, XFD, DCM, ATR.GZ, XFD.GZ "
@@ -114,7 +111,7 @@ class atari800(Runner):
             if callback:
                 callback()
 
-        super(atari800, self).install(version, downloader, on_runner_installed)
+        super().install(version, downloader, on_runner_installed)
 
     def find_good_bioses(self, bios_path):
         """ Check for correct bios files """
