@@ -1,19 +1,22 @@
+# Standard Library
 import os
 import subprocess
 import xml.etree.ElementTree as etree
+from gettext import gettext as _
 
-from lutris.util.log import logger
-from lutris.runners.runner import Runner
+# Lutris Modules
 from lutris import settings
+from lutris.runners.runner import Runner
 from lutris.util import system
+from lutris.util.log import logger
 
 SNES9X_DIR = os.path.join(settings.DATA_DIR, "runners/snes9x")
 
 
 class snes9x(Runner):
-    description = "Super Nintendo emulator"
-    human_name = "Snes9x"
-    platforms = ["Nintendo SNES"]
+    description = _("Super Nintendo emulator")
+    human_name = _("Snes9x")
+    platforms = [_("Nintendo SNES")]
     runnable_alone = True
     runner_executable = "snes9x/bin/snes9x-gtk"
     game_options = [
@@ -21,19 +24,28 @@ class snes9x(Runner):
             "option": "main_file",
             "type": "file",
             "default_path": "game_path",
-            "label": "ROM file",
-            "help": "The game data, commonly called a ROM image.",
+            "label": _("ROM file"),
+            "help": _("The game data, commonly called a ROM image."),
         }
     ]
 
     runner_options = [
-        {"option": "fullscreen", "type": "bool", "label": "Fullscreen", "default": "1"},
         {
-            "option": "maintain_aspect_ratio",
+            "option": "fullscreen",
             "type": "bool",
-            "label": "Maintain aspect ratio (4:3)",
-            "default": "1",
-            "help": (
+            "label": _("Fullscreen"),
+            "default": "1"
+        },
+        {
+            "option":
+            "maintain_aspect_ratio",
+            "type":
+            "bool",
+            "label":
+            _("Maintain aspect ratio (4:3)"),
+            "default":
+            "1",
+            "help": _(
                 "Super Nintendo games were made for 4:3 "
                 "screens with rectangular pixels, but modern screens "
                 "have square pixels, which results in a vertically "
@@ -44,7 +56,7 @@ class snes9x(Runner):
         {
             "option": "sound_driver",
             "type": "choice",
-            "label": "Sound driver",
+            "label": _("Sound driver"),
             "advanced": True,
             "choices": (("SDL", "1"), ("ALSA", "2"), ("OSS", "0")),
             "default": "1",
@@ -54,7 +66,8 @@ class snes9x(Runner):
     def set_option(self, option, value):
         config_file = os.path.expanduser("~/.snes9x/snes9x.xml")
         if not system.path_exists(config_file):
-            subprocess.Popen([self.get_executable(), "-help"])
+            with subprocess.Popen([self.get_executable(), "-help"]) as snes9x_process:
+                snes9x_process.communicate()
         if not system.path_exists(config_file):
             logger.error("Snes9x config file creation failed")
             return
