@@ -1,16 +1,14 @@
 """Options list for system config."""
-# Standard Library
 import glob
-# pylint: disable=invalid-name
 import os
-from collections import OrderedDict
+import subprocess
+from collections import OrderedDict, defaultdict
 from gettext import gettext as _
 
-# Lutris Modules
 from lutris import runners
-from lutris.discord import DiscordPresence
-from lutris.util import system
+from lutris.util import linux, system
 from lutris.util.display import DISPLAY_MANAGER, SCREEN_SAVER_INHIBITOR, USE_DRI_PRIME
+from lutris.util.graphics import drivers
 
 VULKAN_DATA_DIRS = [
     "/usr/local/etc/vulkan",  # standard site-local location
@@ -67,21 +65,80 @@ def get_optirun_choices():
     return choices
 
 
+def get_gpu_vendor_cmd(nvidia_files):
+    """Run glxinfo command to get vendor based on certain conditions"""
+    glxinfocmd = "glxinfo | grep -i opengl | grep -i vendor"
+
+    if USE_DRI_PRIME == 1:
+        glxinfocmd = "DRI_PRIME=1 glxinfo | grep -i opengl | grep -i vendor"
+    elif nvidia_files == 1:
+        glxinfocmd = "__GLX_VENDOR_LIBRARY_NAME=nvidia glxinfo | grep -i opengl | grep -i vendor"
+    return glxinfocmd
+
+
 def get_vk_icd_choices():
     """Return available Vulkan ICD loaders"""
-    choices = [(_("Auto"), "")]
-
+    intel = []
+    amdradv = []
+    nvidia = []
+    amdvlk = []
+    amdvlkpro = []
+    choices = [(_("Auto: WARNING -- No Vulkan Loader detected!"), "")]
+    icd_files = defaultdict(list)
     # Add loaders
     for data_dir in VULKAN_DATA_DIRS:
         path = os.path.join(data_dir, "icd.d", "*.json")
         for loader in glob.glob(path):
-            choices.append((os.path.basename(loader), loader))
+            icd_key = os.path.basename(loader).split(".")[0]
+            icd_files[icd_key].append(os.path.join(path, loader))
+            if "intel" in loader:
+                intel.append(loader)
+            elif "radeon" in loader:
+                amdradv.append(loader)
+            elif "nvidia" in loader:
+                nvidia.append(loader)
+            elif "amd" in loader:
+                if "pro" in loader:
+                    amdvlkpro.append(loader)
+                else:
+                    amdvlk.append(loader)
 
+    intel_files = ":".join(intel)
+    amdradv_files = ":".join(amdradv)
+    nvidia_files = ":".join(nvidia)
+    amdvlk_files = ":".join(amdvlk)
+    amdvlkpro_files = ":".join(amdvlkpro)
+
+    glxinfocmd = get_gpu_vendor_cmd(0)
+    if nvidia_files:
+        glxinfocmd = get_gpu_vendor_cmd(1)
+    with subprocess.Popen(glxinfocmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT) as glxvendorget:
+        glxvendor = glxvendorget.communicate()[0].decode("utf-8")
+    default_gpu = glxvendor
+
+    if "Intel" in default_gpu:
+        choices = [(_("Auto: Intel Open Source (MESA: ANV)"), intel_files)]
+    elif "AMD" in default_gpu:
+        choices = [(_("Auto: AMD RADV Open Source (MESA: RADV)"), amdradv_files)]
+    elif "NVIDIA" in default_gpu:
+        choices = [(_("Auto: Nvidia Proprietary"), nvidia_files)]
+
+    if intel_files:
+        choices.append(("Intel Open Source (MESA: ANV)", intel_files))
+    if amdradv_files:
+        choices.append(("AMD RADV Open Source (MESA: RADV)", amdradv_files))
+    if nvidia_files:
+        choices.append(("Nvidia Proprietary", nvidia_files))
+    if amdvlk_files:
+        if not amdvlkpro_files:
+            choices.append(("AMDVLK/AMDGPU-PRO Proprietary", amdvlk_files))
+        else:
+            choices.append(("AMDVLK Open source", amdvlk_files))
+    if amdvlkpro_files:
+        choices.append(("AMDGPU-PRO Proprietary", amdvlkpro_files))
     return choices
 
 
-discord_presence = DiscordPresence()
-
 system_options = [  # pylint: disable=invalid-name
     {
         "option": "game_path",
@@ -113,37 +170,64 @@ system_options = [  # pylint: disable=invalid-name
                   " over the provided ones."),
     },
     {
-        "option":
-        "reset_desktop",
-        "type":
-        "bool",
-        "label":
-        _("Restore resolution on game exit"),
-        "default":
-        False,
+        "option": "reset_desktop",
+        "type": "bool",
+        "label": _("Restore resolution on game exit"),
+        "default": False,
         "help": _("Some games don't restore your screen resolution when \n"
                   "closed or when they crash. This is when this option comes \n"
                   "into play to save your bacon."),
     },
     {
-        "option": "single_cpu",
+        "option": "gamescope",
         "type": "bool",
-        "label": _("Restrict to single core"),
-        "advanced": True,
+        "label": _("Enable gamescope"),
         "default": False,
-        "help": _("Restrict the game to a single CPU core."),
+        "advanced": True,
+        "condition": bool(system.find_executable("gamescope")) and not drivers.is_nvidia(),
+        "help": _("Use gamescope to draw the game window isolated from your desktop.\n"
+                  "Use Ctrl+Super+F to toggle fullscreen"),
     },
     {
-        "option":
-        "restore_gamma",
-        "type":
-        "bool",
-        "default":
-        False,
-        "label":
-        _("Restore gamma on game exit"),
-        "advanced":
-        True,
+        "option": "gamescope_output_res",
+        "type": "string",
+        "label": _("Gamescope output resolution"),
+        "default": False,
+        "advanced": True,
+        "condition": bool(system.find_executable("gamescope")),
+        "help": _("Resolution of the window on your desktop"),
+    },
+    {
+        "option": "gamescope_game_res",
+        "type": "string",
+        "label": _("Gamescope game resolution"),
+        "default": False,
+        "advanced": True,
+        "condition": bool(system.find_executable("gamescope")),
+        "help": _("Resolution of the screen visible to the game"),
+    },
+    {
+        "option": "single_cpu",
+        "type": "bool",
+        "label": _("Restrict number of cores used"),
+        "advanced": True,
+        "default": False,
+        "help": _("Restrict the game to a maximum number of CPU cores."),
+    },
+    {
+        "option": "limit_cpu_count",
+        "type": "string",
+        "label": _("Restrict number of cores to"),
+        "advanced": True,
+        "default": "1",
+        "help": _("Maximum number of CPU cores to be used, if 'Restrict number of cores used' is turned on."),
+    },
+    {
+        "option": "restore_gamma",
+        "type": "bool",
+        "default": False,
+        "label": _("Restore gamma on game exit"),
+        "advanced": True,
         "help": _("Some games don't correctly restores gamma on exit, making "
                   "your display too bright. Select this option to correct it."),
     },
@@ -195,18 +279,12 @@ system_options = [  # pylint: disable=invalid-name
         "help": _("Switch to US keyboard QWERTY layout while game is running"),
     },
     {
-        "option":
-        "optimus",
-        "type":
-        "choice",
-        "default":
-        "off",
-        "choices":
-        get_optirun_choices,
-        "label":
-        _("Optimus launcher (NVIDIA Optimus laptops)"),
-        "advanced":
-        True,
+        "option": "optimus",
+        "type": "choice",
+        "default": "off",
+        "choices": get_optirun_choices,
+        "label": _("Optimus launcher (NVIDIA Optimus laptops)"),
+        "advanced": True,
         "help": _("If you have installed the primus or bumblebee packages, "
                   "select what launcher will run the game with the command, "
                   "activating your NVIDIA graphic chip for high 3D "
@@ -217,7 +295,7 @@ system_options = [  # pylint: disable=invalid-name
     {
         "option": "vk_icd",
         "type": "choice",
-        "default": "",
+        "default": get_vk_icd_choices()[0][1],
         "choices": get_vk_icd_choices,
         "label": _("Vulkan ICD loader"),
         "advanced": True,
@@ -228,16 +306,9 @@ system_options = [  # pylint: disable=invalid-name
     },
     {
         "option": "mangohud",
-        "type": "choice",
+        "type": "bool",
         "label": _("FPS counter (MangoHud)"),
-        "choices": (
-            (_("Disabled"), ""),
-            (_("Enabled (Vulkan)"), "vk64"),
-            (_("Enabled (OpenGL)"), "gl64"),
-            (_("Enabled (OpenGL, 32bit)"), "gl32")
-        ),
-        "default": "",
-        "advanced": False,
+        "default": False,
         "condition": bool(system.find_executable("mangohud")),
         "help": _("Display the game's FPS + other information. Requires MangoHud to be installed."),
     },
@@ -250,33 +321,20 @@ system_options = [  # pylint: disable=invalid-name
         "condition": bool(system.find_executable("strangle")),
         "help": _("Limit the game's FPS to desired number"),
     },
-    {
-        "option": "aco",
-        "type": "bool",
-        "label": _("Enable ACO shader compiler"),
-        "condition": system.LINUX_SYSTEM.is_feature_supported("ACO"),
-        "help": _("Enable ACO shader compiler, improving performance in a lot of games. "
-                  "Requires Mesa 19.3 or later.")
-    },
     {
         "option": "gamemode",
         "type": "bool",
-        "default": system.LINUX_SYSTEM.gamemode_available(),
-        "condition": system.LINUX_SYSTEM.gamemode_available,
+        "default": linux.LINUX_SYSTEM.gamemode_available(),
+        "condition": linux.LINUX_SYSTEM.gamemode_available(),
         "label": _("Enable Feral GameMode"),
         "help": _("Request a set of optimisations be temporarily applied to the host OS"),
     },
     {
-        "option":
-        "prime",
-        "type":
-        "bool",
-        "default":
-        False,
-        "condition":
-        True,
-        "label":
-        _("Enable NVIDIA Prime Render Offload"),
+        "option": "prime",
+        "type": "bool",
+        "default": False,
+        "condition": True,
+        "label": _("Enable NVIDIA Prime Render Offload"),
         "help": _("If you have the latest NVIDIA driver and the properly patched xorg-server (see "
                   "https://download.nvidia.com/XFree86/Linux-x86_64/435.17/README/primerenderoffload.html"
                   "), you can launch a game on your NVIDIA GPU by toggling this switch. This will apply "
@@ -284,53 +342,36 @@ system_options = [  # pylint: disable=invalid-name
                   "__GLX_VENDOR_LIBRARY_NAME=nvidia environment variables.")
     },
     {
-        "option":
-        "dri_prime",
-        "type":
-        "bool",
-        "default":
-        USE_DRI_PRIME,
-        "condition":
-        USE_DRI_PRIME,
-        "label":
-        _("Use discrete graphics"),
-        "advanced":
-        True,
+        "option": "dri_prime",
+        "type": "bool",
+        "default": USE_DRI_PRIME,
+        "condition": USE_DRI_PRIME,
+        "label": _("Use discrete graphics"),
+        "advanced": True,
         "help": _("If you have open source graphic drivers (Mesa), selecting this "
                   "option will run the game with the 'DRI_PRIME=1' environment variable, "
                   "activating your discrete graphic chip for high 3D "
                   "performance."),
     },
     {
-        "option":
-        "sdl_video_fullscreen",
-        "type":
-        "choice",
-        "label":
-        _("SDL 1.2 Fullscreen Monitor"),
-        "choices":
-        get_output_list,
-        "default":
-        "off",
-        "advanced":
-        True,
+        "option": "sdl_video_fullscreen",
+        "type": "choice",
+        "label": _("SDL 1.2 Fullscreen Monitor"),
+        "choices": get_output_list,
+        "default": "off",
+        "advanced": True,
         "help": _("Hint SDL 1.2 games to use a specific monitor when going "
                   "fullscreen by setting the SDL_VIDEO_FULLSCREEN "
                   "environment variable"),
     },
     {
-        "option":
-        "display",
-        "type":
-        "choice",
-        "label":
-        _("Turn off monitors except"),
-        "choices":
-        get_output_choices,
-        "default":
-        "off",
-        "advanced":
-        True,
+        "option": "display",
+        "type": "choice",
+        "label": _("Turn off monitors except"),
+        "choices": get_output_choices,
+        "condition": linux.LINUX_SYSTEM.display_server != "wayland",
+        "default": "off",
+        "advanced": True,
         "help": _("Only keep the selected screen active while the game is "
                   "running. \n"
                   "This is useful if you have a dual-screen setup, and are \n"
@@ -341,34 +382,29 @@ system_options = [  # pylint: disable=invalid-name
         "type": "choice",
         "label": _("Switch resolution to"),
         "choices": get_resolution_choices,
+        "condition": linux.LINUX_SYSTEM.display_server != "wayland",
         "default": "off",
         "help": _("Switch to this screen resolution while the game is running."),
     },
     {
         "option": "terminal",
-        "label": _("Run in a terminal"),
+        "label": _("CLI mode"),
         "type": "bool",
         "default": False,
         "advanced": True,
-        "help": _("Run the game in a new terminal window."),
+        "help": _("Enable a terminal for text-based games. "
+                  "Only useful for ASCII based games. May cause issues with graphical games."),
     },
     {
-        "option":
-        "terminal_app",
-        "label":
-        _("Terminal application"),
-        "type":
-        "choice_with_entry",
-        "choices":
-        system.get_terminal_apps,
-        "default":
-        system.get_default_terminal(),
-        "advanced":
-        True,
-        "help": _("The terminal emulator to be run with the previous option. "
+        "option": "terminal_app",
+        "label": _("Text based games emulator"),
+        "type": "choice_with_entry",
+        "choices": linux.get_terminal_apps,
+        "default": linux.get_default_terminal(),
+        "advanced": True,
+        "help": _("The terminal emulator used with the CLI mode. "
                   "Choose from the list of detected terminal apps or enter "
-                  "the terminal's command or path. "
-                  "Note: Not all terminal emulators are guaranteed to work."),
+                  "the terminal's command or path."),
     },
     {
         "option": "env",
@@ -376,6 +412,13 @@ system_options = [  # pylint: disable=invalid-name
         "label": _("Environment variables"),
         "help": _("Environment variables loaded at run time"),
     },
+    {
+        "option": "antimicro_config",
+        "type": "file",
+        "label": _("AntiMicroX Profile"),
+        "advanced": True,
+        "help": _("Path to an AntiMicroX profile file"),
+    },
     {
         "option": "prefix_command",
         "type": "string",
@@ -414,28 +457,20 @@ system_options = [  # pylint: disable=invalid-name
         "help": _("Script to execute when the game exits"),
     },
     {
-        "option":
-        "include_processes",
-        "type":
-        "string",
-        "label":
-        _("Include processes"),
-        "advanced":
-        True,
+        "option": "include_processes",
+        "type": "string",
+        "label": _("Include processes"),
+        "advanced": True,
         "help": _("What processes to include in process monitoring. "
                   "This is to override the built-in exclude list.\n"
                   "Space-separated list, processes including spaces "
                   "can be wrapped in quotation marks."),
     },
     {
-        "option":
-        "exclude_processes",
-        "type":
-        "string",
-        "label":
-        _("Exclude processes"),
-        "advanced":
-        True,
+        "option": "exclude_processes",
+        "type": "string",
+        "label": _("Exclude processes"),
+        "advanced": True,
         "help": _("What processes to exclude in process monitoring. "
                   "For example background processes that stick around "
                   "after the game has been closed.\n"
@@ -443,49 +478,35 @@ system_options = [  # pylint: disable=invalid-name
                   "can be wrapped in quotation marks."),
     },
     {
-        "option":
-        "killswitch",
-        "type":
-        "string",
-        "label":
-        _("Killswitch file"),
-        "advanced":
-        True,
+        "option": "killswitch",
+        "type": "string",
+        "label": _("Killswitch file"),
+        "advanced": True,
         "help": _("Path to a file which will stop the game when deleted \n"
                   "(usually /dev/input/js0 to stop the game on joystick "
                   "unplugging)"),
     },
     {
-        "option":
-        "sdl_gamecontrollerconfig",
-        "type":
-        "string",
-        "label":
-        _("SDL2 gamepad mapping"),
-        "advanced":
-        True,
+        "option": "sdl_gamecontrollerconfig",
+        "type": "string",
+        "label": _("SDL2 gamepad mapping"),
+        "advanced": True,
         "help": _("SDL_GAMECONTROLLERCONFIG mapping string or path to a custom "
                   "gamecontrollerdb.txt file containing mappings."),
     },
     {
-        "option":
-        "xephyr",
-        "label":
-        _("Use Xephyr"),
-        "type":
-        "choice",
+        "option": "xephyr",
+        "label": _("Use Xephyr"),
+        "type": "choice",
         "choices": (
             (_("Off"), "off"),
             (_("8BPP (256 colors)"), "8bpp"),
             (_("16BPP (65536 colors)"), "16bpp"),
             (_("24BPP (16M colors)"), "24bpp"),
         ),
-        "default":
-        "off",
-        "advanced":
-        True,
-        "help":
-        _("Run program in Xephyr to support 8BPP and 16BPP color modes"),
+        "default": "off",
+        "advanced": True,
+        "help": _("Run program in Xephyr to support 8BPP and 16BPP color modes"),
     },
     {
         "option": "xephyr_resolution",
@@ -504,49 +525,6 @@ system_options = [  # pylint: disable=invalid-name
     },
 ]
 
-discord_options = [
-    {
-        "option": "discord_rpc_enabled",
-        "type": "bool",
-        "label": _("Discord Rich Presence"),
-        "default": False,
-        "condition": discord_presence.available,
-        "help": _("Enable status to Discord of this game being played"),
-    },
-    {
-        "option": "discord_show_runner",
-        "type": "bool",
-        "label": _("Discord Show Runner"),
-        "default": True,
-        "condition": discord_presence.available,
-        "help": _("Embed the runner name in the Discord status"),
-    },
-    {
-        "option": "discord_custom_game_name",
-        "type": "string",
-        "label": _("Discord Custom Game Name"),
-        "condition": discord_presence.available,
-        "help": _("Custom name to override with and pass to Discord"),
-    },
-    {
-        "option": "discord_custom_runner_name",
-        "type": "string",
-        "label": _("Discord Custom Runner Name"),
-        "condition": discord_presence.available,
-        "help": _("Custom runner name to override with and pass to Discord"),
-    },
-    {
-        "option": "discord_client_id",
-        "type": "string",
-        "label": _("Discord Client ID"),
-        "condition": discord_presence.available,
-        "help": _("Custom Discord Client ID for passing status"),
-    },
-]
-
-if discord_presence.available:
-    system_options += discord_options
-
 
 def with_runner_overrides(runner_slug):
     """Return system options updated with overrides from given runner."""
