@@ -5,7 +5,13 @@ import os
 import subprocess
 import gi
 
-gi.require_version("GnomeDesktop", "3.0")
+try:
+    gi.require_version("GnomeDesktop", "3.0")
+    from gi.repository import GnomeDesktop
+    LIB_GNOME_DESKTOP_AVAILABLE = True
+except ValueError:
+    LIB_GNOME_DESKTOP_AVAILABLE = False
+    GnomeDesktop = None
 
 try:
     from dbus.exceptions import DBusException
@@ -13,7 +19,7 @@ try:
 except ImportError:
     DBUS_AVAILABLE = False
 
-from gi.repository import Gdk, GLib, GnomeDesktop, Gio
+from gi.repository import Gdk, GLib, Gio, Gtk
 
 from lutris.util import system
 from lutris.util.graphics.displayconfig import MutterDisplayManager
@@ -26,11 +32,21 @@ class NoScreenDetected(Exception):
     """Raise this when unable to detect screens"""
 
 
+def get_default_dpi():
+    """Computes the DPI to use for the primary monitor
+    which we pass to WINE."""
+    display = Gdk.Display.get_default()
+    monitor = display.get_primary_monitor()
+    scale = monitor.get_scale_factor()
+    dpi = 96 * scale
+    return int(dpi)
+
+
 def restore_gamma():
     """Restores gamma to a normal level."""
     xgamma_path = system.find_executable("xgamma")
     try:
-        subprocess.Popen([xgamma_path, "-gamma", "1.0"])
+        subprocess.Popen([xgamma_path, "-gamma", "1.0"])  # pylint: disable=consider-using-with
     except (FileNotFoundError, TypeError):
         logger.warning("xgamma is not available on your system")
     except PermissionError:
@@ -57,10 +73,12 @@ def _get_graphics_adapters():
 
 
 class DisplayManager:
-
     """Get display and resolution using GnomeDesktop"""
 
     def __init__(self):
+        if not LIB_GNOME_DESKTOP_AVAILABLE:
+            logger.warning("libgnomedesktop unavailable")
+            return
         screen = Gdk.Screen.get_default()
         if not screen:
             raise NoScreenDetected
@@ -124,10 +142,12 @@ def get_display_manager():
             logger.exception("Failed to instanciate MutterDisplayConfig. Please report with exception: %s", ex)
     else:
         logger.error("DBus is not available, lutris was not properly installed.")
-    try:
-        return DisplayManager()
-    except (GLib.Error, NoScreenDetected):
-        return LegacyDisplayManager()
+    if LIB_GNOME_DESKTOP_AVAILABLE:
+        try:
+            return DisplayManager()
+        except (GLib.Error, NoScreenDetected):
+            pass
+    return LegacyDisplayManager()
 
 
 DISPLAY_MANAGER = get_display_manager()
@@ -167,7 +187,7 @@ def get_desktop_environment():
 def _get_command_output(*command):
     """Some rogue function that gives no shit about residing in the correct module"""
     try:
-        return subprocess.Popen(
+        return subprocess.Popen(  # pylint: disable=consider-using-with
             command,
             stdin=subprocess.DEVNULL,
             stdout=subprocess.PIPE,
@@ -237,7 +257,11 @@ def _run_command(*command):
     are you lost little _run_command?
     """
     try:
-        return subprocess.Popen(command, stdin=subprocess.DEVNULL, close_fds=True)
+        return subprocess.Popen(  # pylint: disable=consider-using-with
+            command,
+            stdin=subprocess.DEVNULL,
+            close_fds=True
+        )
     except FileNotFoundError:
         logger.error("Oh no")
 
@@ -260,6 +284,7 @@ def disable_compositing():
 def enable_compositing():
     """Re-enable compositing if the corresponding call to disable_compositing
     disabled it."""
+
     compositing_disabled = _COMPOSITING_DISABLED_STACK.pop()
     if not compositing_disabled:
         return
@@ -271,9 +296,20 @@ def enable_compositing():
 class DBusScreenSaverInhibitor:
 
     """Inhibit and uninhibit the screen saver using DBus.
-    Requires the Inhibit() and UnInhibit() methods to be exposed over DBus."""
 
-    def __init__(self, name, path, interface, bus_type=Gio.BusType.SESSION):
+    It will use the Gtk.Application's inhibit and uninhibit methods to inhibit
+    the screen saver.
+
+    For enviroments which don't support either org.freedesktop.ScreenSaver or
+    org.gnome.ScreenSaver interfaces one can declare a DBus interface which
+    requires the Inhibit() and UnInhibit() methods to be exposed."""
+
+    def __init__(self):
+        self.proxy = None
+
+    def set_dbus_iface(self, name, path, interface, bus_type=Gio.BusType.SESSION):
+        """Sets a dbus proxy to be used instead of Gtk.Application methods, this
+        method can raise an exception."""
         self.proxy = Gio.DBusProxy.new_for_bus_sync(
             bus_type, Gio.DBusProxyFlags.NONE, None, name, path, interface, None)
 
@@ -281,37 +317,68 @@ class DBusScreenSaverInhibitor:
         """Inhibit the screen saver.
         Returns a cookie that must be passed to the corresponding uninhibit() call.
         If an error occurs, None is returned instead."""
-        try:
-            return self.proxy.Inhibit("(ss)", "Lutris", "Running game: %s" % game_name)
-        except Exception:
-            return None
+        reason = "Running game: %s" % game_name
+
+        if self.proxy:
+            try:
+                return self.proxy.Inhibit("(ss)", "Lutris", reason)
+            except Exception:
+                return None
+        else:
+            app = Gio.Application.get_default()
+            window = app.window
+            flags = Gtk.ApplicationInhibitFlags.SUSPEND | Gtk.ApplicationInhibitFlags.IDLE
+            cookie = app.inhibit(window, flags, reason)
+
+            # Gtk.Application.inhibit returns 0 if there was an error.
+            if cookie == 0:
+                return None
+
+            return cookie
 
     def uninhibit(self, cookie):
         """Uninhibit the screen saver.
         Takes a cookie as returned by inhibit. If cookie is None, no action is taken."""
-        if cookie is not None:
+        if not cookie:
+            return
+
+        if self.proxy:
             self.proxy.UnInhibit("(u)", cookie)
+        else:
+            app = Gio.Application.get_default()
+            app.uninhibit(cookie)
 
 
 def _get_screen_saver_inhibitor():
     """Return the appropriate screen saver inhibitor instance.
-    Returns None if the required interface isn't available."""
+    If the required interface isn't available, it will default to GTK's
+    implementation."""
     desktop_environment = get_desktop_environment()
+
+    name = None
+    inhibitor = DBusScreenSaverInhibitor()
+
     if desktop_environment is DesktopEnvironment.MATE:
         name = "org.mate.ScreenSaver"
         path = "/"
+        interface = "org.mate.ScreenSaver"
     elif desktop_environment is DesktopEnvironment.XFCE:
+        # According to
+        # https://github.com/xfce-mirror/xfce4-session/blob/master/xfce4-session/xfce-screensaver.c#L240
+        # The XFCE enviroment does support the org.freedesktop.ScreenSaver interface
+        # but this might be not present in older releases.
         name = "org.xfce.ScreenSaver"
         path = "/"
-    else:
-        name = "org.freedesktop.ScreenSaver"
-        path = "/org/freedesktop/ScreenSaver"
-    interface = name
-    try:
-        return DBusScreenSaverInhibitor(name, path, interface)
-    except GLib.Error as err:
-        logger.error("Error during creation of DBusScreenSaverInhibitor: %s", err.message)  # pylint: disable=no-member
-        return None
+        interface = "org.xfce.ScreenSaver"
+
+    if name:
+        try:
+            inhibitor.set_dbus_iface(name, path, interface)
+        except GLib.Error as err:
+            logger.warning("Failed to set up a DBus proxy for name %s, path %s, "
+                           "interface %s: %s", name, path, interface, str(err))
+
+    return inhibitor
 
 
 SCREEN_SAVER_INHIBITOR = _get_screen_saver_inhibitor()
