@@ -1,5 +1,3 @@
-# Standard Library
-import errno
 import gzip
 import os
 import shutil
@@ -8,14 +6,12 @@ import tarfile
 import uuid
 import zlib
 
-# Lutris Modules
 from lutris import settings
 from lutris.util import system
 from lutris.util.log import logger
 
 
 class ExtractFailure(Exception):
-
     """Exception raised when and archive fails to extract"""
 
 
@@ -75,14 +71,16 @@ def is_7zip_supported(path, extractor):
 
 def guess_extractor(path):
     """Guess what extractor should be used from a file name"""
-    if path.endswith(".tar.gz") or path.endswith(".tgz"):
-        extractor = "tgz"
-    elif path.endswith(".tar.xz") or path.endswith(".txz"):
-        extractor = "txz"
-    elif path.endswith(".tar"):
+    if path.endswith(".tar"):
         extractor = "tar"
-    elif path.endswith(".tar.bz2") or path.endswith(".tbz"):
-        extractor = "bz2"
+    elif path.endswith((".tar.gz", ".tgz")):
+        extractor = "tgz"
+    elif path.endswith((".tar.xz", ".txz", ".tar.lzma")):
+        extractor = "txz"
+    elif path.endswith((".tar.bz2", ".tbz2", ".tbz")):
+        extractor = "tbz2"
+    elif path.endswith((".tar.zst", ".tzst")):
+        extractor = "tzst"
     elif path.endswith(".gz"):
         extractor = "gzip"
     elif path.endswith(".exe"):
@@ -94,17 +92,19 @@ def guess_extractor(path):
     return extractor
 
 
-def get_archive_opener(extractor, path):
+def get_archive_opener(extractor):
     """Return the archive opener and optional mode for an extractor"""
     mode = None
-    if extractor == "tgz":
+    if extractor == "tar":
+        opener, mode = tarfile.open, "r:"
+    elif extractor == "tgz":
         opener, mode = tarfile.open, "r:gz"
     elif extractor == "txz":
         opener, mode = tarfile.open, "r:xz"
-    elif extractor == "tar":
-        opener, mode = tarfile.open, "r:"
-    elif extractor == "bz2":
+    elif extractor == "tbz2":
         opener, mode = tarfile.open, "r:bz2"
+    elif extractor == "tzst":
+        opener, mode = tarfile.open, "r:zst"  # Note: not supported by tarfile yet
     elif extractor == "gzip":
         opener = "gz"
     elif extractor == "gog":
@@ -125,14 +125,14 @@ def extract_archive(path, to_directory=".", merge_single=True, extractor=None):
     if extractor is None:
         extractor = guess_extractor(path)
 
-    opener, mode = get_archive_opener(extractor, path)
+    opener, mode = get_archive_opener(extractor)
 
     temp_path = temp_dir = os.path.join(to_directory, ".extract-%s" % random_id())
     try:
         _do_extract(path, temp_path, opener, mode, extractor)
     except (OSError, zlib.error, tarfile.ReadError, EOFError) as ex:
         logger.error("Extraction failed: %s", ex)
-        raise ExtractFailure(str(ex))
+        raise ExtractFailure(str(ex)) from ex
     if merge_single:
         extracted = os.listdir(temp_path)
         if len(extracted) == 1:
@@ -168,7 +168,7 @@ def extract_archive(path, to_directory=".", merge_single=True, extractor=None):
                             destination_path,
                             ex,
                         )
-                        raise ExtractFailure(str(ex))
+                        raise ExtractFailure(str(ex)) from ex
             else:
                 shutil.move(source_path, destination_path)
         system.remove_folder(temp_dir)
@@ -216,16 +216,21 @@ def extract_deb(archive, dest):
     extract_7zip(archive, dest, archive_type="ar")
     debian_folder = os.path.join(dest, "debian")
     os.makedirs(debian_folder)
-    shutil.move(os.path.join(dest, "control.tar.gz"), debian_folder)
-    data_file = os.path.join(dest, "data.tar.gz")
-    extractor = "r:gz"
-    if not os.path.exists(data_file):
-        data_file = os.path.join(dest, "data.tar.xz")
-        extractor = "r:xz"
-    handler = tarfile.open(data_file, extractor)
-    handler.extractall(dest)
-    handler.close()
-    os.remove(data_file)
+
+    control_file_exts = [".gz", ".xz", ".zst", ""]
+    for extension in control_file_exts:
+        control_tar_path = os.path.join(dest, "control.tar{}".format(extension))
+        if os.path.exists(control_tar_path):
+            shutil.move(control_tar_path, debian_folder)
+            break
+
+    data_file_exts = [".gz", ".xz", ".zst", ".bz2", ".lzma", ""]
+    for extension in data_file_exts:
+        data_tar_path = os.path.join(dest, "data.tar{}".format(extension))
+        if os.path.exists(data_tar_path):
+            extract_archive(data_tar_path, dest)
+            os.remove(data_tar_path)
+            break
 
 
 def extract_gog(path, dest):
@@ -235,34 +240,42 @@ def extract_gog(path, dest):
         raise RuntimeError("specified exe is not a GOG setup file")
 
 
-# Check if exe is in fact an inno setup file
+def get_innoextract_path():
+    """Return the path where innoextract is installed"""
+    inno_dirs = [path for path in os.listdir(settings.RUNTIME_DIR) if path.startswith("innoextract")]
+    if inno_dirs:
+        inno_path = os.path.join(settings.RUNTIME_DIR, inno_dirs[0], "innoextract")
+    else:
+        inno_path = system.find_executable("innoextract")
+        if inno_path:
+            logger.warning("innoextract not available in the runtime folder, using some random version")
+    if system.path_exists(inno_path):
+        return inno_path
+
+
 def check_inno_exe(path):
-    _innoextract_path = os.path.join(settings.RUNTIME_DIR, "innoextract/innoextract")
-    if not system.path_exists(_innoextract_path):
-        _innoextract_path = system.find_executable("innoextract")
-    if not system.path_exists(_innoextract_path):
-        return False  # Can't find innoextract
+    """Check if a path in a compatible innosetup archive"""
+    _innoextract_path = get_innoextract_path()
+    if not _innoextract_path:
+        logger.warning("Innoextract not found, can't determine type of archive %s", path)
+        return False
     command = [_innoextract_path, "-i", path]
     return_code = subprocess.call(command)
-    if return_code != 0:
-        return False
-    return True
+    return return_code == 0
+
+
+def get_innoextract_list(file_path):
+    """Return the list of files contained in a GOG archive"""
+    output = system.read_process_output([get_innoextract_path(), "-lmq", file_path])
+    return [line[3:] for line in output.split("\n") if line]
 
 
 def decompress_gog(file_path, destination_path):
-    _innoextract_path = os.path.join(settings.RUNTIME_DIR, "innoextract/innoextract")
-    if not system.path_exists(_innoextract_path):
-        _innoextract_path = system.find_executable("innoextract")
-    if not system.path_exists(_innoextract_path):
+    innoextract_path = get_innoextract_path()
+    if not innoextract_path:
         raise OSError("innoextract is not found in the lutris runtime or on the system")
-    # innoextract cannot do mkdir -p, so we have to do it here:
-    try:
-        os.makedirs(destination_path)
-    except OSError as e:
-        if e.errno != errno.EEXIST:
-            raise OSError("cannot make output directory for extracting setup file")
-    command = [_innoextract_path, "-g", "-d", destination_path, "-e", file_path]
-    return_code = subprocess.call(command)
+    system.create_folder(destination_path)  # innoextract cannot do mkdir -p
+    return_code = subprocess.call([innoextract_path, "-m", "-g", "-d", destination_path, "-e", file_path])
     if return_code != 0:
         raise RuntimeError("innoextract failed to extract GOG setup file")
 
@@ -273,8 +286,7 @@ def decompress_gz(file_path, dest_path):
         dest_filename = os.path.join(dest_path, os.path.basename(file_path[:-3]))
     else:
         dest_filename = file_path[:-3]
-    if not os.path.exists(os.path.dirname(dest_filename)):
-        os.makedirs(os.path.dirname(dest_filename))
+    os.makedirs(os.path.dirname(dest_filename), exist_ok=True)
 
     with open(dest_filename, "wb") as dest_file:
         gzipped_file = gzip.open(file_path, "rb")
