@@ -17,7 +17,7 @@ def get_nvidia_driver_info():
     version_file_path = "/proc/driver/nvidia/version"
     if not os.path.exists(version_file_path):
         return
-    with open(version_file_path) as version_file:
+    with open(version_file_path, encoding='utf-8') as version_file:
         content = version_file.readlines()
         nvrm_version = content[0].split(': ')[1].strip().split()
         return {
@@ -39,7 +39,7 @@ def get_nvidia_gpu_ids():
 
 def get_nvidia_gpu_info(gpu_id):
     """Return details about a GPU"""
-    with open("/proc/driver/nvidia/gpus/%s/information" % gpu_id) as info_file:
+    with open("/proc/driver/nvidia/gpus/%s/information" % gpu_id, encoding='utf-8') as info_file:
         content = info_file.readlines()
     infos = {}
     for line in content:
@@ -67,7 +67,7 @@ def get_gpu_info(card):
     """Return information about a GPU"""
     infos = {"DRIVER": "", "PCI_ID": "", "PCI_SUBSYS_ID": ""}
     try:
-        with open("/sys/class/drm/%s/device/uevent" % card) as card_uevent:
+        with open("/sys/class/drm/%s/device/uevent" % card, encoding='utf-8') as card_uevent:
             content = card_uevent.readlines()
     except FileNotFoundError:
         logger.error("Unable to read driver information for card %s", card)
