@@ -19,7 +19,7 @@ class AsyncCall(threading.Thread):
         self.source_id = None
         self.stop_request = threading.Event()
 
-        super(AsyncCall, self).__init__(target=self.target, args=args, kwargs=kwargs)
+        super().__init__(target=self.target, args=args, kwargs=kwargs)
         self.function = func
         self.callback = callback if callback else lambda r, e: None
         self.daemon = kwargs.pop("daemon", True)
@@ -33,10 +33,9 @@ class AsyncCall(threading.Thread):
         try:
             result = self.function(*args, **kwargs)
         except Exception as ex:  # pylint: disable=broad-except
-            logger.error("Error while completing task %s: %s", self.function, ex)
+            logger.error("Error while completing task %s: %s %s", self.function, type(ex), ex)
             error = ex
-            ex_type, ex_value, trace = sys.exc_info()
-            print(ex_type, ex_value)
+            _ex_type, _ex_value, trace = sys.exc_info()
             traceback.print_tb(trace)
 
         self.source_id = GLib.idle_add(self.callback, result, error)
