@@ -1,5 +1,6 @@
 """System utilities"""
 import hashlib
+import inspect
 import os
 import re
 import shutil
@@ -7,18 +8,23 @@ import signal
 import stat
 import string
 import subprocess
+from gettext import gettext as _
 
-from gi.repository import GLib
+from gi.repository import Gio, GLib
 
 from lutris import settings
-from lutris.util.linux import LINUX_SYSTEM
 from lutris.util.log import logger
 
-# Home folders that should never get deleted. This should be localized and return the
-# appropriate folders names for the current locale.
+# Home folders that should never get deleted.
 PROTECTED_HOME_FOLDERS = (
-    "Documents", "Downloads", "Desktop",
-    "Pictures", "Videos", "Pictures", "Projects", "Games"
+    _("Documents"),
+    _("Downloads"),
+    _("Desktop"),
+    _("Pictures"),
+    _("Videos"),
+    _("Pictures"),
+    _("Projects"),
+    _("Games")
 )
 
 
@@ -60,14 +66,16 @@ def execute(command, env=None, cwd=None, log_errors=False, quiet=False, shell=Fa
     # Piping stderr can cause slowness in the programs, use carefully
     # (especially when using regedit with wine)
     try:
-        stdout, stderr = subprocess.Popen(
+        with subprocess.Popen(
             command,
             shell=shell,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE if log_errors else subprocess.DEVNULL,
             env=existing_env,
             cwd=cwd,
-        ).communicate(timeout=timeout)
+            errors="replace"
+        ) as command_process:
+            stdout, stderr = command_process.communicate(timeout=timeout)
     except (OSError, TypeError) as ex:
         logger.error("Could not run command %s (env: %s): %s", command, env, ex)
         return ""
@@ -76,7 +84,21 @@ def execute(command, env=None, cwd=None, log_errors=False, quiet=False, shell=Fa
         return ""
     if stderr and log_errors:
         logger.error(stderr)
-    return stdout.decode(errors="replace").strip()
+    return stdout.strip()
+
+
+def read_process_output(command, timeout=5):
+    """Return the output of a command as a string"""
+    try:
+        return subprocess.check_output(
+            command,
+            timeout=timeout,
+            encoding="utf-8",
+            errors="ignore"
+        ).strip()
+    except (OSError, subprocess.CalledProcessError, subprocess.TimeoutExpired) as ex:
+        logger.error("%s command failed: %s", command, ex)
+        return ""
 
 
 def get_md5_hash(filename):
@@ -115,9 +137,6 @@ def find_executable(exec_name):
     """Return the absolute path of an executable"""
     if not exec_name:
         return None
-    cached = LINUX_SYSTEM.get(exec_name)
-    if cached:
-        return cached
     return shutil.which(exec_name)
 
 
@@ -177,7 +196,7 @@ def substitute(string_template, variables):
     identifiers = variables.keys()
 
     # We support dashes in identifiers but they are not valid in python
-    # identifers, which is a requirement for the templating engine we use
+    # identifiers, which is a requirement for the templating engine we use
     # Replace the dashes with underscores in the mapping and template
     variables = dict((k.replace("-", "_"), v) for k, v in variables.items())
     for identifier in identifiers:
@@ -192,22 +211,12 @@ def substitute(string_template, variables):
 def merge_folders(source, destination):
     """Merges the content of source to destination"""
     logger.debug("Merging %s into %s", source, destination)
-    source = os.path.abspath(source)
-    for (dirpath, dirnames, filenames) in os.walk(source):
-        source_relpath = dirpath[len(source):].strip("/")
-        dst_abspath = os.path.join(destination, source_relpath)
-        for dirname in dirnames:
-            new_dir = os.path.join(dst_abspath, dirname)
-            logger.debug("creating dir: %s", new_dir)
-            try:
-                os.mkdir(new_dir)
-            except OSError:
-                pass
-        for filename in filenames:
-            # logger.debug("Copying %s", filename)
-            if not os.path.exists(dst_abspath):
-                os.makedirs(dst_abspath)
-            shutil.copy(os.path.join(dirpath, filename), os.path.join(dst_abspath, filename))
+    # Check if dirs_exist_ok is defined ( Python >= 3.8)
+    sig = inspect.signature(shutil.copytree)
+    if "dirs_exist_ok" in sig.parameters:
+        shutil.copytree(source, destination, symlinks=False, ignore_dangling_symlinks=True, dirs_exist_ok=True)
+    else:
+        shutil.copytree(source, destination, symlinks=False, ignore_dangling_symlinks=True)
 
 
 def remove_folder(path):
@@ -233,11 +242,21 @@ def create_folder(path):
     if not path:
         return
     path = os.path.expanduser(path)
-    if not os.path.exists(path):
-        os.makedirs(path)
+    os.makedirs(path, exist_ok=True)
     return path
 
 
+def list_unique_folders(folders):
+    """Deduplicate directories with the same Device.Inode"""
+    unique_dirs = {}
+    for folder in folders:
+        folder_stat = os.stat(folder)
+        identifier = "%s.%s" % (folder_stat.st_dev, folder_stat.st_ino)
+        if identifier not in unique_dirs:
+            unique_dirs[identifier] = folder
+    return unique_dirs.values()
+
+
 def is_removeable(path):
     """Check if a folder is safe to remove (not system or home, ...)"""
     if not path_exists(path):
@@ -257,28 +276,26 @@ def is_removeable(path):
 
 
 def fix_path_case(path):
-    """Do a case insensitive check, return the real path with correct case."""
+    """Do a case insensitive check, return the real path with correct case. If the path is
+    not for a real file, this corrects as many components as do exist."""
     if not path or os.path.exists(path):
         # If a path isn't provided or it exists as is, return it.
         return path
     parts = path.strip("/").split("/")
     current_path = "/"
     for part in parts:
-        if not os.path.exists(current_path):
-            return
-        tested_path = os.path.join(current_path, part)
-        if os.path.exists(tested_path):
-            current_path = tested_path
-            continue
-        try:
-            path_contents = os.listdir(current_path)
-        except OSError:
-            logger.error("Can't read contents of %s", current_path)
-            path_contents = []
-        for filename in path_contents:
-            if filename.lower() == part.lower():
-                current_path = os.path.join(current_path, filename)
-                continue
+        parent_path = current_path
+        current_path = os.path.join(current_path, part)
+        if not os.path.exists(current_path) and os.path.isdir(parent_path):
+            try:
+                path_contents = os.listdir(parent_path)
+            except OSError:
+                logger.error("Can't read contents of %s", parent_path)
+                path_contents = []
+            for filename in path_contents:
+                if filename.lower() == part.lower():
+                    current_path = os.path.join(parent_path, filename)
+                    break
 
     # Only return the path if we got the same number of elements
     if len(parts) == len(current_path.strip("/").split("/")):
@@ -298,19 +315,6 @@ def get_pids_using_file(path):
     return set(fuser_output.split())
 
 
-def get_terminal_apps():
-    """Return the list of installed terminal emulators"""
-    return LINUX_SYSTEM.get_terminals()
-
-
-def get_default_terminal():
-    """Return the default terminal emulator"""
-    terms = get_terminal_apps()
-    if terms:
-        return terms[0]
-    logger.error("Couldn't find a terminal emulator.")
-
-
 def reverse_expanduser(path):
     """Replace '/home/username' with '~' in given path."""
     if not path:
@@ -352,19 +356,6 @@ def reset_library_preloads():
                 logger.error("Failed to delete environment variable %s", key)
 
 
-def run_once(function):
-    """Decorator to use on functions intended to run only once"""
-    first_run = True
-
-    def fn_wrapper(*args):
-        nonlocal first_run
-        if first_run:
-            first_run = False
-            return function(*args)
-
-    return fn_wrapper
-
-
 def get_existing_parent(path):
     """Return the 1st existing parent for a folder (or itself if the path
     exists and is a directory). returns None, when none of the parents exists.
@@ -395,3 +386,54 @@ def get_disk_size(path):
             if os.path.isfile(os.path.join(base, f))
         ])
     return total_size
+
+
+def get_running_pid_list():
+    """Return the list of PIDs from processes currently running"""
+    return [int(p) for p in os.listdir("/proc") if p[0].isdigit()]
+
+
+def get_mounted_discs():
+    """Return a list of mounted discs and ISOs
+
+    :rtype: list of Gio.Mount
+    """
+    volumes = Gio.VolumeMonitor.get()
+    drives = []
+
+    for mount in volumes.get_mounts():
+        if mount.get_volume():
+            device = mount.get_volume().get_identifier("unix-device")
+            if not device:
+                logger.debug("No device for mount %s", mount.get_name())
+                continue
+
+            # Device is a disk drive or ISO image
+            if "/dev/sr" in device or "/dev/loop" in device:
+                drives.append(mount.get_root().get_path())
+    return drives
+
+
+def find_mount_point(path):
+    """Return the mount point a file is located on"""
+    path = os.path.abspath(path)
+    while not os.path.ismount(path):
+        path = os.path.dirname(path)
+    return path
+
+
+def get_mountpoint_drives():
+    """Return a mapping of mount points with their corresponding drives"""
+    mounts = read_process_output(["mount", "-v"]).split("\n")
+    mount_map = []
+    for mount in mounts:
+        mount_parts = mount.split()
+        if len(mount_parts) < 3:
+            continue
+        mount_map.append((mount_parts[2], mount_parts[0]))
+    return dict(mount_map)
+
+
+def get_drive_for_path(path):
+    """Return the physical drive a file is located on"""
+    return get_mountpoint_drives().get(find_mount_point(path))
